{% extends "../core/document.html" %}

{% comment ***************** %}
{% comment Emit content      %}
{% comment ***************** %}
{% block doc-content %}

	{% comment ************************ %}
	{% comment Emit model mosaics       %}
	{% comment ************************ %}
	{% for i, mosaic in enumerate(doc.esm_model_grids) %}
		{% set mosaic_id = i %}
		{% set header = "Model Grid: " +  mosaic.short_name %}
		{% include grids_grid_mosaic.html %}
	{% end %}

	{% comment ************************ %}
	{% comment Emit exchange mosaics    %}
	{% comment ************************ %}
	{% for i, mosaic in enumerate(doc.esm_exchange_grids) %}
		{% set mosaic_id = i %}
		{% set header = "Exchange Grid: " +  mosaic.short_name %}
		{% include grids_grid_mosaic.html %}
	{% end %}

{% end %}