<section id="{{ c.meta.id }}" class="esdoc-nav-target">
	{% comment ************************************************** %}
	{% comment Breadcrumb 	 		    			  			  %}
	{% comment ************************************************** %}
	<header>
		<h3>
		{% for pc in c.ext.ancestors[1:] %}
			<span id="breadcrumb-{{ pc.meta.id }}">
			{{ pc.ext.short_display_name }} >
			</span>
		{% end %}
		{{ c.ext.short_display_name }}
		</h3>
	</header>

	{% comment ************************************************** %}
	{% comment Sub-section: overview 	 		    			  %}
	{% comment ************************************************** %}
	{% if c.description and len(c.description) %}
	<section>
		<header>
			<h4>Overview</h4>
		</header>
		<p>{{ c.description }}</p>
	</section>
	{% end %}

	{% comment ************************************************** %}
	{% comment Sub-section: scientific properties 	 		      %}
	{% comment ************************************************** %}
	{% if c.ext.displayable_scientific_properties %}
	<section>
		<header>
			<h4>Scientific Properties ({{ len(c.ext.displayable_scientific_properties) }})</h4>
		</header>
		<table>
			<tbody>
				{% for p in c.ext.displayable_scientific_properties %}
					{% for v in sorted(p.values) %}
					<tr class="esdoc-field">
						<td>
							<b>{{ " > ".join(p.ext.display_name.split(" > ")[1:]) }}</b>: {{v}}
						</td>
					</tr>
					{% end %}
				{% end %}
			</tbody>
		</table>
	</section>
    {% end %}

	{% comment ************************************************** %}
	{% comment Sub-section: quality control properties 	 		  %}
	{% comment ************************************************** %}
	{% if c.ext.displayable_qc_properties %}
	<section>
		<header>
			<h4>Component Documentation Quality Control</h4>
		</header>
		<table>
			<tbody>
				{% for p in c.ext.displayable_qc_properties %}
					{% for v in sorted(p.values) %}
					<tr class="esdoc-field">
						<td>
							<b>{{ " > ".join(p.ext.display_name.split(" > ")[1:]) }}</b>:
							{% if p.ext.short_display_name == "Verification Status" %}
								{% if v == "0" %}
									Awaiting review
								{% elif v == "1" %}
									Under review
								{% elif v == "2" %}
									Accepted
								{% end %}
							{% else %}
								{{v}}
						    {% end %}
						</td>
					</tr>
					{% end %}
				{% end %}
			</tbody>
		</table>
	</section>
    {% end %}

	{% comment ************************************************** %}
	{% comment Sub-section: citations 		    				  %}
	{% comment ************************************************** %}
	{% set previousCtx = ctx %}
	{% set citations = c.citations %}
	{% set header = 4 %}
	{% include shared_citation.html %}
	{% set ctx = previousCtx %}

	{% comment **************** %}
	{% comment ... contacts     %}
	{% comment **************** %}
    {% set contacts = [rp for rp in c.responsible_parties if rp.role.lower() not in ['funder', 'centre']] %}
	{% set previousCtx = ctx %}
	{% set header = 4 %}
	{% include shared_responsible_party.html %}
	{% set ctx = previousCtx %}

</section>
