{% comment ***************** %}
{% comment Emit overview     %}
{% comment ***************** %}
<section>
    <header>
        <h4>Overview</h4>
    </header>
    <table>
        <tbody>
        {% set ctx = TemplateInfo(mosaic, fieldset='cim.1.grids.grid-mosaic') %}
        {% include ../core/fieldset.html %}
        </tbody>
    </table>
</section>

{% comment ***************** %}
{% comment Emit tiles        %}
{% comment ***************** %}
{% for tile in mosaic.tiles %}
    {% include grids_grid_tile.html %}
{% end %}
