<section>
	<header>
		<h4>Tile {{ resolution_type }} Resolution</h4>
	</header>
	<table>
		<tbody>
		{% set ctx = TemplateInfo(resolution, fieldset=('cim.1.grids.grid-tile-resolution', resolution)) %}
		{% include ../core/fieldset.html %}
		</tbody>
	</table>
</section>
