"""
.. module:: pyesdoc.mp.ontologies.schemas.cim.__init__.py
   :license: GPL/CeCIL
   :platform: Unix, Windows
   :synopsis: Sub-package initializer.

.. moduleauthor:: Mark Conway-Greenslade <momipsl@ipsl.jussieu.fr>


"""
from pyesdoc.mp.ontologies.schemas.cim import v1
from pyesdoc.mp.ontologies.schemas.cim import v2