{% extends "../core/document.html" %}

{% comment ***************** %}
{% comment Emit content      %}
{% comment ***************** %}
{% block doc-content %}

	{% include data_data_object_overview.html %}
	{% if doc.citations %}
		{% set citations = doc.citations %}
		{% set header = 4 %}
		{% include shared_citation.html %}
	{% end %}

{% end %}