{% extends "../core/document.html" %}

{% comment ******************* %}
{% comment Emit navigation     %}
{% comment ******************* %}
{% block doc-nav %}
<nav>
    <ul>
	    <li><a href="#{{ doc.ext.full_id }}-details">Details</a></li>
		{% if doc.inputs %}
		    <li><a href="#{{ doc.ext.full_id }}-inputs">Couplings</a></li>
	    {% end %}
		{% if isinstance(doc.model, cim.v1.DocReference) and doc.model.changes %}
		    <li><a href="#{{ doc.ext.full_id }}-model-mods">Model Modifications</a></li>
		{% end %}
    </ul>
</nav>
{% end %}

{% comment ***************** %}
{% comment Emit content      %}
{% comment ***************** %}
{% block doc-content %}

	{% include activity_simulation_run_details.html %}
	{% if doc.inputs %}
		{% include activity_simulation_run_inputs.html %}
	{% end %}
	{% if isinstance(doc.model, cim.v1.DocReference) and doc.model.changes %}
		{% include activity_simulation_run_model_mods.html %}
	{% end %}

{% end %}