# -*- coding: utf-8 -*-
"""
.. module:: pyesdoc.ontologies.__init__.py

   :license: GPL/CeCIL
   :platform: Unix, Windows
   :synopsis: Ontologies sub-package init.

.. moduleauthor:: Mark Conway-Greenslade <momipsl@ipsl.jussieu.fr>


"""
import type_info
from utils import *
