<section id="{{ doc.ext.full_id }}-details">
	<header>
		<h3>Details</h3>
	</header>

	{% comment ************************* %}
	{% comment Sub-section: overview     %}
	{% comment ************************* %}
	<section>
		<header>
			<h4>Overview</h4>
		</header>
		<table>
			<tbody>
			{% set ctx = TemplateInfo(doc, fieldset='cim.1.activity.simulationrun-overview') %}
			{% include ../core/fieldset.html %}
			</tbody>
		</table>
	</section>

	{% comment ************************* %}
	{% comment Sub-section: contacts    %}
	{% comment ************************* %}
	{% if doc.responsible_parties %}
		{% set contacts = doc.responsible_parties %}
		{% set header = 4 %}
		{% include shared_responsible_party.html %}
	{% end %}

</section>