{% comment ******************* %}
{% comment Emit tile overview  %}
{% comment ******************* %}
<section>
	<header>
		<h4>Tile Overview</h4>
	</header>
	<table>
		<tbody>
		{% set ctx = TemplateInfo(tile, fieldset='cim.1.grids.grid-tile-overview') %}
		{% include ../core/fieldset.html %}
		</tbody>
	</table>
</section>

{% comment ******************* %}
{% comment Emit tile extent    %}
{% comment ******************* %}
{% if tile.extent %}
<section>
	<header>
		<h4>Tile Extent</h4>
	</header>
	<table>
		<tbody>
		{% set ctx = TemplateInfo(tile.extent, fieldset='cim.1.grids.grid-tile-extent') %}
		{% include ../core/fieldset.html %}
		</tbody>
	</table>
</section>
{% end %}

{% comment ******************* %}
{% comment Emit tile hor. res  %}
{% comment ******************* %}
{% if tile.horizontal_resolution %}
	{% set resolution = tile.horizontal_resolution %}
	{% set resolution_type = "Horizontal" %}
	{% include grids_grid_tile_resolution.html %}
{% end %}

{% comment ******************* %}
{% comment Emit tile ver. res  %}
{% comment ******************* %}
{% if tile.vertical_resolution %}
	{% set resolution = tile.vertical_resolution %}
	{% set resolution_type = "Vertical" %}
	{% include grids_grid_tile_resolution.html %}
{% end %}
